Package: urltools
Type: Package
Title: Vectorised Tools for URL Handling and Parsing
Version: 1.7.3.1
Date: 2019-04-14
Authors@R: c(person(given = "Os",
                    family = "Keyes",
                    role = c("aut", "cre"),
                    email = "ironholds@gmail.com"),
             person(given = "Jay",
                    family = "Jacobs",
                    role = "aut"),
             person(given = "Drew",
                    family = "Schmidt",
                    role = "aut"),
             person(given = "Mark",
                    family = "Greenaway",
                    role = "ctb"),
             person(given = "Bob",
                    family = "Rudis",
                    role = "ctb"),
             person(given = "Alex",
                    family = "Pinto",
                    role = "ctb"),
             person(given = "Maryam",
                    family = "Khezrzadeh",
                    role = "ctb"),
             person(given = "Peter",
                    family = "Meilstrup",
                    role = "ctb"),
             person(given = c("Adam", "M."),
                    family = "Costello",
                    role = "cph"),
             person(given = "Jeff",
                    family = "Bezanson",
                    role = "cph"),
             person(given = "Peter",
                    family = "Meilstrup",
                    role = "ctb"),
             person(given = "Xueyuan",
                    family = "Jiang",
                    role = "ctb"))
Description: A toolkit for all URL-handling needs, including encoding and decoding,
    parsing, parameter extraction and modification. All functions are
    designed to be both fast and entirely vectorised. It is intended to be
    useful for people dealing with web-related datasets, such as server-side
    logs, although may be useful for other situations involving large sets of
    URLs.
License: MIT + file LICENSE
LazyData: TRUE
LinkingTo: Rcpp
Imports: Rcpp, methods, triebeard
Suggests: testthat, knitr
URL: https://github.com/Ironholds/urltools/
BugReports: https://github.com/Ironholds/urltools/issues
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Encoding: UTF-8
Depends: R (>= 2.10)
NeedsCompilation: yes
Packaged: 2025-06-12 07:48:37 UTC; hornik
Repository: CRAN
Date/Publication: 2025-06-12 08:37:42 UTC
Author: Os Keyes [aut, cre],
  Jay Jacobs [aut],
  Drew Schmidt [aut],
  Mark Greenaway [ctb],
  Bob Rudis [ctb],
  Alex Pinto [ctb],
  Maryam Khezrzadeh [ctb],
  Peter Meilstrup [ctb],
  Adam M. Costello [cph],
  Jeff Bezanson [cph],
  Peter Meilstrup [ctb],
  Xueyuan Jiang [ctb]
Maintainer: Os Keyes <ironholds@gmail.com>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 05:51:32 UTC; unix
Archs: urltools.so.dSYM

Package: urltools
Type: Package
Title: Vectorised Tools for URL Handling and Parsing
Version: 1.7.3.1
Date: 2019-04-14
Authors@R: c(person(given = "Os",
                    family = "Keyes",
                    role = c("aut", "cre"),
                    email = "ironholds@gmail.com"),
             person(given = "Jay",
                    family = "Jacobs",
                    role = "aut"),
             person(given = "Drew",
                    family = "Schmidt",
                    role = "aut"),
             person(given = "Mark",
                    family = "Greenaway",
                    role = "ctb"),
             person(given = "Bob",
                    family = "Rudis",
                    role = "ctb"),
             person(given = "Alex",
                    family = "Pinto",
                    role = "ctb"),
             person(given = "Maryam",
                    family = "Khezrzadeh",
                    role = "ctb"),
             person(given = "Peter",
                    family = "Meilstrup",
                    role = "ctb"),
             person(given = c("Adam", "M."),
                    family = "Costello",
                    role = "cph"),
             person(given = "Jeff",
                    family = "Bezanson",
                    role = "cph"),
             person(given = "Peter",
                    family = "Meilstrup",
                    role = "ctb"),
             person(given = "Xueyuan",
                    family = "Jiang",
                    role = "ctb"))
Description: A toolkit for all URL-handling needs, including encoding and decoding,
    parsing, parameter extraction and modification. All functions are
    designed to be both fast and entirely vectorised. It is intended to be
    useful for people dealing with web-related datasets, such as server-side
    logs, although may be useful for other situations involving large sets of
    URLs.
License: MIT + file LICENSE
LazyData: TRUE
LinkingTo: Rcpp
Imports: Rcpp, methods, triebeard
Suggests: testthat, knitr
URL: https://github.com/Ironholds/urltools/
BugReports: https://github.com/Ironholds/urltools/issues
VignetteBuilder: knitr
RoxygenNote: 6.1.1
Encoding: UTF-8
Depends: R (>= 2.10)
NeedsCompilation: yes
Packaged: 2025-06-12 07:48:37 UTC; hornik
Repository: CRAN
Date/Publication: 2025-06-12 08:37:42 UTC
Author: Os Keyes [aut, cre],
  Jay Jacobs [aut],
  Drew Schmidt [aut],
  Mark Greenaway [ctb],
  Bob Rudis [ctb],
  Alex Pinto [ctb],
  Maryam Khezrzadeh [ctb],
  Peter Meilstrup [ctb],
  Adam M. Costello [cph],
  Jeff Bezanson [cph],
  Peter Meilstrup [ctb],
  Xueyuan Jiang [ctb]
Maintainer: Os Keyes <ironholds@gmail.com>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 23:21:11 UTC; unix
Archs: urltools.so.dSYM

# Generated by roxygen2: do not edit by hand

export("domain<-")
export("fragment<-")
export("parameters<-")
export("path<-")
export("port<-")
export("scheme<-")
export(domain)
export(fragment)
export(get_credentials)
export(host_extract)
export(param_get)
export(param_remove)
export(param_set)
export(parameters)
export(path)
export(port)
export(puny_decode)
export(puny_encode)
export(scheme)
export(strip_credentials)
export(suffix_extract)
export(suffix_refresh)
export(tld_extract)
export(tld_refresh)
export(url_compose)
export(url_decode)
export(url_encode)
export(url_parse)
import(methods)
importFrom(Rcpp,sourceCpp)
importFrom(triebeard,longest_match)
importFrom(triebeard,trie)
useDynLib(urltools, .registration = TRUE)

## urltools
A package for elegantly handling and parsing URLs from within R.

__Author:__ Oliver Keyes, Jay Jacobs<br/>
__License:__ [MIT](http://opensource.org/licenses/MIT)<br/>
__Status:__ Stable

[![Travis-CI Build Status](https://travis-ci.org/Ironholds/urltools.svg?branch=master)](https://travis-ci.org/Ironholds/urltools) ![downloads](http://cranlogs.r-pkg.org/badges/grand-total/urltools)

### Description

URLs in R are often treated as nothing more than part of data retrieval -
they're used for making connections and reading data. With web analytics
and research, however, URLs can *be* the data, and R's default handlers
are not best suited to handle vectorised operations over large datasets.
<code>urltools</code> is intended to solve this. 

It contains drop-in replacements for R's URLdecode and URLencode functions, along
with new functionality such as a URL parser and parameter value extractor. In all
cases, the functions are designed to be content-safe (not breaking on unexpected values)
and fully vectorised, resulting in a dramatic speed improvement over existing implementations -
crucial for large datasets. For more information, see the [urltools vignette](https://github.com/Ironholds/urltools/blob/master/vignettes/urltools.Rmd).

Please note that this project is released with a [Contributor Code of Conduct](https://github.com/Ironholds/urltools/blob/master/CODE_OF_CONDUCT.md).
By participating in this project you agree to abide by its terms.

### Installation

The latest CRAN version can be obtained via:

    install.packages("urltools")
    
To get the development version:

    devtools::install_github("ironholds/urltools")

### Dependencies
* R. Doy.
* [Rcpp](https://cran.r-project.org/package=Rcpp)

## ----eval=FALSE---------------------------------------------------------------
# URLdecode("test%gIL")
# Error in rawToChar(out) : embedded nul in string: '\0L'
# In addition: Warning message:
# In URLdecode("%gIL") : out-of-range values treated as 0 in coercion to raw

## ----eval=FALSE---------------------------------------------------------------
# URLencode("https://en.wikipedia.org/wiki/Article", reserved = TRUE)
# [1] "https%3a%2f%2fen.wikipedia.org%2fwiki%2fArticle"

## ----eval=FALSE---------------------------------------------------------------
# library(urltools)
# url_decode("test%gIL")
# [1] "test"
# url_encode("https://en.wikipedia.org/wiki/Article")
# [1] "https://en.wikipedia.org%2fwiki%2fArticle"

## ----eval=FALSE---------------------------------------------------------------
# > parsed_address <- url_parse("https://en.wikipedia.org/wiki/Article")
# > str(parsed_address)
# 'data.frame':	1 obs. of  6 variables:
#  $ scheme   : chr "https"
#  $ domain   : chr "en.wikipedia.org"
#  $ port     : chr NA
#  $ path     : chr "wiki/Article"
#  $ parameter: chr NA
#  $ fragment : chr NA

## ----eval=FALSE---------------------------------------------------------------
# > url_compose(parsed_address)
# [1] "https://en.wikipedia.org/wiki/article"

## ----eval=FALSE---------------------------------------------------------------
# url <- "https://en.wikipedia.org/wiki/Article"
# scheme(url)
# "https"
# scheme(url) <- "ftp"
# url
# "ftp://en.wikipedia.org/wiki/Article"

## ----eval=FALSE---------------------------------------------------------------
# > url <- "https://en.wikipedia.org/wiki/Article"
# > domain_name <- domain(url)
# > domain_name
# [1] "en.wikipedia.org"
# > str(suffix_extract(domain_name))
# 'data.frame':	1 obs. of  4 variables:
#  $ host     : chr "en.wikipedia.org"
#  $ subdomain: chr "en"
#  $ domain   : chr "wikipedia"
#  $ suffix      : chr "org"

## ----eval=FALSE---------------------------------------------------------------
# domain_name <- domain("https://en.wikipedia.org/wiki/Article")
# updated_suffixes <- suffix_refresh()
# suffix_extract(domain_name, updated_suffixes)

## ----eval=FALSE---------------------------------------------------------------
# domain_name <- domain("https://en.wikipedia.org/wiki/Article")
# host_extract(domain_name)

## ----eval=FALSE---------------------------------------------------------------
# > str(param_get(urls = "http://en.wikipedia.org/wiki/api.php?action=parse&pageid=1023&export=json",
#                      parameter_names = c("pageid","export")))
# 'data.frame':	1 obs. of  2 variables:
#  $ pageid: chr "1023"
#  $ export: chr "json"

## ----eval=FALSE---------------------------------------------------------------
# url <- "http://en.wikipedia.org/wiki/api.php?action=parse&pageid=1023&export=json"
# url <- param_set(url, key = "pageid", value = "12")
# url
# # [1] "http://en.wikipedia.org/wiki/api.php?action=parse&pageid=12&export=json"

## ----eval=FALSE---------------------------------------------------------------
# url <- "http://en.wikipedia.org/wiki/api.php"
# url <- param_set(url, key = "pageid", value = "12")
# url
# # [1] "http://en.wikipedia.org/wiki/api.php?pageid=12"

## ----eval=FALSE---------------------------------------------------------------
# url <- "http://en.wikipedia.org/wiki/api.php?action=parse&pageid=1023&export=json"
# url <- param_remove(url, keys = c("action","export"))
# url
# # [1] "http://en.wikipedia.org/wiki/api.php?pageid=1023"

#' @title Tools for handling URLs
#' @name urltools
#' @description This package provides functions for URL encoding and decoding,
#' parsing, and parameter extraction, designed to be both fast and
#' entirely vectorised. It is intended to be useful for people dealing with
#' web-related datasets, such as server-side logs.
#' 
#' @seealso the \href{https://CRAN.R-project.org/package=urltools/vignettes/urltools.html}{package vignette}.
#' @useDynLib urltools, .registration = TRUE
#' @importFrom Rcpp sourceCpp
#' @docType package
#' @aliases urltools urltools-package
NULL

urltools_env <- new.env(parent = emptyenv())

suffix_load <- function(suffixes = NULL){
  if(is.null(suffixes)){
    suffixes <- urltools::suffix_dataset
  }
  cleaned_suffixes <- gsub(x = suffixes$suffixes, pattern = "*.", replacement = "", fixed = TRUE)
  is_wildcard <- cleaned_suffixes[which(grepl(x = suffixes$suffixes, pattern = "*.", fixed = TRUE))]
  suff_trie <- triebeard::trie(keys = reverse_strings(paste0(".", cleaned_suffixes)),
                               values = cleaned_suffixes)
  return(list(suff_trie = suff_trie,
              is_wildcard = is_wildcard,
              cleaned_suffixes = cleaned_suffixes))
  return(invisible())
}

.onLoad <- function(...) {
  holding <- suffix_load()
  assign("is_wildcard", holding$is_wildcard, envir = urltools_env)
  assign("cleaned_suffixes", holding$cleaned_suffixes, envir = urltools_env)
  assign("suff_trie", holding$suff_trie, envir = urltools_env)
}

#'@title Get or set a URL's scheme
#'@description as in the lubridate package, individual components of a URL
#'can be both extracted or set using the relevant function call - see the
#'examples.
#'@aliases scheme
#'@rdname scheme
#'
#'@param x a URL, or vector of URLs
#'
#'@param value a replacement value (or vector of replacement values)
#'for x's scheme.
#'
#'@seealso \code{\link{domain}}, \code{\link{port}}, \code{\link{path}},
#'\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
#'
#'@examples
#'#Get a component
#'example_url <- "http://cran.r-project.org/submit.html"
#'scheme(example_url)
#'
#'#Set a component
#'scheme(example_url) <- "https"
#'
#'# NA out the URL
#'scheme(example_url) <- NA_character_
#'@import methods
#'@export
scheme <- function(x){
  return(get_component_(x,0))
}

"scheme<-" <- function(x, value) standardGeneric("scheme<-")
#'@rdname scheme
#'@export
setGeneric("scheme<-", useAsDefault = function(x, value){
  if(length(value) == 0 && is.null(value)){
    return(rm_component_(x, 0))
  }
  return(set_component_r(x, 0, value, "://"))
})

#'@title Get or set a URL's domain
#'@description as in the lubridate package, individual components of a URL
#'can be both extracted or set using the relevant function call - see the
#'examples.
#'@aliases domain
#'@rdname domain
#'
#'@param x a URL, or vector of URLs
#'
#'@param value a replacement value (or vector of replacement values)
#'for x's scheme.
#'
#'@seealso \code{\link{scheme}}, \code{\link{port}}, \code{\link{path}},
#'\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
#'
#'@examples
#'#Get a component
#'example_url <- "http://cran.r-project.org/submit.html"
#'domain(example_url)
#'
#'#Set a component
#'domain(example_url) <- "en.wikipedia.org"
#'@export
domain <- function(x){
  return(get_component_(x,1))
}
"domain<-" <- function(x, value) standardGeneric("domain<-")
#'@rdname domain
#'@export
setGeneric("domain<-", useAsDefault = function(x, value){
  if(length(value) == 0 && is.null(value)){
    return(rm_component_(x, 1))
  }
  return(set_component_(x, 1, value))
})

#'@title Get or set a URL's port
#'@description as in the lubridate package, individual components of a URL
#'can be both extracted or set using the relevant function call - see the
#'examples.
#'
#'@aliases port
#'@rdname port
#'
#'@param x a URL, or vector of URLs
#'
#'@param value a replacement value (or vector of replacement values)
#'for x's port. If NULL, the port will be entirely removed.
#'
#'@seealso \code{\link{scheme}}, \code{\link{domain}}, \code{\link{path}},
#'\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
#'
#'@examples
#'# Get the port
#'example_url <- "http://cran.r-project.org:80/submit.html"
#'port(example_url)
#'
#'# Set the port
#'port(example_url) <- "12"
#'
#'# Remove the port
#'port(example_url) <- NULL
#'@export
port <- function(x){
  return(get_component_(x,2))
}
"port<-" <- function(x, value) standardGeneric("port<-")
#'@rdname port
#'@export
setGeneric("port<-", useAsDefault = function(x, value){
  if(length(value) == 0 && is.null(value)){
    return(rm_component_(x, 2))
  }
  return(set_component_f(x, 2, value, ":"))
})

#'@title Get or set a URL's path
#'@description as in the lubridate package, individual components of a URL
#'can be both extracted or set using the relevant function call - see the
#'examples.
#'@aliases path
#'@rdname path
#'
#'@param x a URL, or vector of URLs
#'
#'@param value a replacement value (or vector of replacement values)
#'for x's path. If NULL, the path will be removed entirely.
#'
#'@seealso \code{\link{scheme}}, \code{\link{domain}}, \code{\link{port}},
#'\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
#'
#'@examples
#'# Get the path
#'example_url <- "http://cran.r-project.org:80/submit.html"
#'path(example_url)
#'
#'# Set the path
#'path(example_url) <- "bin/windows/"
#'
#'# Remove the path
#'path(example_url) <- NULL
#'@export
path <- function(x){
  return(get_component_(x,3))
}
"path<-" <- function(x, value) standardGeneric("path<-")
#'@rdname path
#'@export
setGeneric("path<-", useAsDefault = function(x, value){
  if(length(value) == 0 && is.null(value)){
    return(rm_component_(x, 3))
  }
  return(set_component_f(x, 3, value, "/"))
})

#'@title Get or set a URL's parameters
#'@description as in the lubridate package, individual components of a URL
#'can be both extracted or set using the relevant function call - see the
#'examples.
#'
#'@aliases parameters
#'@rdname parameters
#'
#'@param x a URL, or vector of URLs
#'
#'@param value a replacement value (or vector of replacement values)
#'for x's parameters. If NULL, the parameters will be removed entirely.
#'
#'@seealso \code{\link{scheme}}, \code{\link{domain}}, \code{\link{port}},
#'\code{\link{path}} and \code{\link{fragment}} for other accessors.
#'
#'@examples
#'# Get the parameters
#'example_url <- "http://en.wikipedia.org/wiki/Aaron_Halfaker?debug=true"
#'parameters(example_url)
#'
#'# Set the parameters
#'parameters(example_url) <- "debug=false"
#'
#'# Remove the parameters
#'parameters(example_url) <- NULL
#'@export
parameters <- function(x){
  return(get_component_(x,4))
}
"parameters<-" <- function(x, value) standardGeneric("parameters<-")
#'@rdname parameters
#'@export
setGeneric("parameters<-", useAsDefault = function(x, value){
  if(length(value) == 0 && is.null(value)){
    return(rm_component_(x, 4))
  }
  return(set_component_f(x, 4, value, "?"))
})

#'@title Get or set a URL's fragment
#'@description as in the lubridate package, individual components of a URL
#'can be both extracted or set using the relevant function call - see the
#'examples.
#'@aliases fragment
#'@rdname fragment
#'
#'@param x a URL, or vector of URLs
#'
#'@param value a replacement value (or vector of replacement values)
#'for x's fragment. If NULL, the fragment will be removed entirely.
#'
#'@seealso \code{\link{scheme}}, \code{\link{domain}}, \code{\link{port}},
#'\code{\link{path}} and \code{\link{parameters}} for other accessors.
#'
#'@examples
#'#Get a component
#'example_url <- "http://en.wikipedia.org/wiki/Aaron_Halfaker?debug=true#test"
#'fragment(example_url)
#'
#'#Set a component
#'fragment(example_url) <- "production"
#'
#'#Remove a component
#'fragment(example_url) <- NULL
#'@export
#'@rdname fragment
#'@export
fragment <- function(x){
  return(get_component_(x,5))
}

"fragment<-" <- function(x, value) standardGeneric("fragment<-")
#'@rdname fragment
#'@export
setGeneric("fragment<-", useAsDefault = function(x, value){
  if(length(value) == 0 && is.null(value)){
    return(rm_component_(x, 5))
  }
  return(set_component_f(x, 5, value, "#"))
})

context("URL parsing tests")

test_that("Check parsing identifies each RfC element", {
  
  data <- url_parse("https://www.google.com:80/foo.php?api_params=turnip#ending")
  expect_that(ncol(data), equals(6))
  expect_that(names(data), equals(c("scheme","domain","port","path","parameter","fragment")))
  expect_that(data$scheme[1], equals("https"))
  expect_that(data$domain[1], equals("www.google.com"))
  expect_that(data$port[1], equals("80"))
  expect_that(data$path[1], equals("foo.php"))
  expect_that(data$parameter[1], equals("api_params=turnip"))
  expect_that(data$fragment[1], equals("ending"))
})

test_that("Check parsing can handle missing elements", {
  
  data <- url_parse("https://www.google.com/foo.php?api_params=turnip#ending")
  expect_that(ncol(data), equals(6))
  expect_that(names(data), equals(c("scheme","domain","port","path","parameter","fragment")))
  expect_that(data$scheme[1], equals("https"))
  expect_that(data$domain[1], equals("www.google.com"))
  expect_true(is.na(data$port[1]))
  expect_that(data$path[1], equals("foo.php"))
  expect_that(data$parameter[1], equals("api_params=turnip"))
  expect_that(data$fragment[1], equals("ending"))
})

test_that("Parsing does not up and die and misplace the fragment",{
  data <- url_parse("http://www.yeastgenome.org/locus/S000005366/overview#protein")
  expect_that(data$fragment[1], equals("protein"))
})

test_that("Composing works",{
  url <- c("http://foo.bar.baz/qux/", "https://en.wikipedia.org:4000/wiki/api.php")
  amended_url <- url_compose(url_parse(url))
  expect_that(url, equals(amended_url))
})

test_that("Port handling works", {
  url <- "https://en.wikipedia.org:4000/wiki/api.php"
  expect_that(port(url), equals("4000"))
  expect_that(path(url), equals("wiki/api.php"))
  url <- "https://en.wikipedia.org:4000"
  expect_that(port(url), equals("4000"))
  expect_true(is.na(path(url)))
  url <- "https://en.wikipedia.org:4000/"
  expect_that(port(url), equals("4000"))
  expect_true(is.na(path(url)))
  url <- "https://en.wikipedia.org:4000?foo=bar"
  expect_that(port(url), equals("4000"))
  expect_true(is.na(path(url)))
  expect_that(parameters(url), equals("foo=bar"))
})

test_that("Port handling does not break path handling", {
  url <- "https://en.wikipedia.org/wiki/File:Vice_City_Public_Radio_(logo).jpg"
  expect_true(is.na(port(url)))
  expect_that(path(url), equals("wiki/File:Vice_City_Public_Radio_(logo).jpg"))
})

test_that("URLs with parameters but no paths work", {
  url <- url_parse("http://www.nextpedition.com?inav=menu_travel_nextpedition")
  expect_true(url$domain[1] == "www.nextpedition.com")
  expect_true(is.na(url$port[1]))
  expect_true(is.na(url$path[1]))
  expect_true(url$parameter[1] == "inav=menu_travel_nextpedition")
})

test_that("URLs with user credentials drop said credentials when parsing", {
  out <- urltools::url_parse("http://foo:bar@97.77.104.22:3128")
  testthat::expect_identical(out$domain, "97.77.104.22")
})

test_that("IPv6 URLs can be handled", {
  url <- url_parse("tcp://[2607:5300:61:44f::]:8333")
  expect_true(url$domain[1] == "2607:5300:61:44f::")
  expect_true(url$port[1] == "8333")
})

test_that("URLs with missing paths and parameters, but with fragments, work", {
  url <- urltools::url_parse("http://some.website.com#frag")
  expect_true(url$fragment[1] == "frag")
})

context("Component get/set tests")

test_that("Check elements can be retrieved", {
  url <- "https://www.google.com:80/foo.php?api_params=turnip#ending"
  testthat::expect_equal(scheme(url), "https")
  testthat::expect_equal(domain(url), "www.google.com")
  testthat::expect_equal(port(url), "80")
  testthat::expect_equal(path(url), "foo.php")
  testthat::expect_equal(parameters(url), "api_params=turnip")
  testthat::expect_equal(fragment(url), "ending")
})

test_that("Check elements can be retrieved with NAs", {
  url <- as.character(NA)
  testthat::expect_equal(is.na(scheme(url)), TRUE)
  testthat::expect_equal(is.na(domain(url)), TRUE)
  testthat::expect_equal(is.na(port(url)), TRUE)
  testthat::expect_equal(is.na(path(url)), TRUE)
  testthat::expect_equal(is.na(parameters(url)), TRUE)
  testthat::expect_equal(is.na(fragment(url)), TRUE)
})

test_that("Check elements can be set", {
  url <- "https://www.google.com:80/foo.php?api_params=turnip#ending"
  scheme(url) <- "http"
  testthat::expect_equal(scheme(url), "http")
  domain(url) <- "www.wikipedia.org"
  testthat::expect_equal(domain(url), "www.wikipedia.org")
  port(url) <- "23"
  testthat::expect_equal(port(url), "23")
  path(url) <- "bar.php"
  testthat::expect_equal(path(url), "bar.php")
  parameters(url) <- "api_params=manic"
  testthat::expect_equal(parameters(url), "api_params=manic")
  fragment(url) <- "beginning"
  testthat::expect_equal(fragment(url), "beginning")
})

test_that("Check elements can be set with NAs", {
  url <- "https://www.google.com:80/"
  scheme(url) <- "http"
  testthat::expect_equal(scheme(url), "http")
  domain(url) <- "www.wikipedia.org"
  testthat::expect_equal(domain(url), "www.wikipedia.org")
  port(url) <- "23"
  testthat::expect_equal(port(url), "23")
  path(url) <- "bar.php"
  testthat::expect_equal(path(url), "bar.php")
  parameters(url) <- "api_params=manic"
  testthat::expect_equal(parameters(url), "api_params=manic")
  fragment(url) <- "beginning"
  testthat::expect_equal(fragment(url), "beginning")
})

test_that("Assigning NA with get will NA a URL", {
  url <- "https://www.google.com:80/"
  port(url) <- NA_character_
  testthat::expect_true(is.na(url))
})

test_that("Removing components with a NULL works", {
  url <- "https://www.google.com:80/foo.php?api_params=turnip#ending"
  fragment(url) <- NULL
  testthat::expect_equal(url,
                         "https://www.google.com:80/foo.php?api_params=turnip")
  parameters(url) <- NULL
  testthat::expect_equal(url,
                         "https://www.google.com:80/foo.php")
  path(url) <- NULL
  testthat::expect_equal(url,
                         "https://www.google.com:80")
  port(url) <- NULL
  testthat::expect_equal(url, "https://www.google.com")
})

test_that("Removing non-removable components throws an error", {
  
  url <- "https://en.wikipedia.org/foo.php"
  testthat::expect_error({
    scheme(url) <- NULL
  })
  testthat::expect_error({
    domain(url) <- NULL
  })
})

test_that("Check multiple elements can be set", {
  url <- "https://www.google.com:80/foo.php?api_params=turnip#ending"
  url <- c(url, url)
  scheme(url) <- c("http", "ftp")
  testthat::expect_equal(scheme(url), c("http", "ftp"))
  domain(url) <- c("www.wikipedia.org", "google.com")
  testthat::expect_equal(domain(url), c("www.wikipedia.org",
                                        "google.com"))
  port(url) <- c("23", "86")
  testthat::expect_equal(port(url), c("23", "86"))
  path(url) <- c("bar.php", "baz.html")
  testthat::expect_equal(path(url), c("bar.php", "baz.html"))
  parameters(url) <- c("api_params=manic", "api_params=street")
  testthat::expect_equal(parameters(url), c("api_params=manic", "api_params=street"))
  fragment(url) <- c("beginning", "end")
  testthat::expect_equal(fragment(url), c("beginning", "end"))
})

test_that("Check elements can be set with extraneous separators", {
  url <- "https://www.wikipedia.org:80/bar.php?api_params=manic#beginning"
  backup <- url
  scheme(url) <- "https://"
  testthat::expect_equal(url, backup)
  port(url) <- ":80"
  testthat::expect_equal(url, backup)
  path(url) <- "/bar.php"
  testthat::expect_equal(url, backup)
  parameters(url) <- "?api_params=manic"
  testthat::expect_equal(url, backup)
  fragment(url) <- "#beginning"
  testthat::expect_equal(url, backup)
})

#' @title Dataset of public suffixes
#' @description This dataset contains a registry of public suffixes, as retrieved from
#' and defined by the \href{https://publicsuffix.org/}{public suffix list}. It is
#' sorted by how many periods(".") appear in the suffix, to optimise it for
#' \code{\link{suffix_extract}}.  It is a data.frame with two columns, the first is
#' the list of suffixes and the second is our best guess at the comment or owner 
#' associated with the particular suffix. 
#'
#' @docType data
#' @keywords datasets
#' @name suffix_dataset
#'
#' @seealso \code{\link{suffix_extract}} for extracting suffixes from domain names,
#' and \code{\link{suffix_refresh}} for getting a new, totally-up-to-date dataset
#' version.
#'
#' @usage data(suffix_dataset)
#' @note Last updated 2016-07-31.
#' @format A data.frame of 8030 rows and 2 columns
"suffix_dataset"

#'@title Retrieve a public suffix dataset
#'
#'@description \code{urltools} comes with an inbuilt
#'dataset of public suffixes, \code{\link{suffix_dataset}}.
#'This is used in \code{\link{suffix_extract}} to identify the top-level domain
#'within a particular domain name.
#'
#'While updates to the dataset will be included in each new package release,
#'there's going to be a gap between changes to the suffixes list and changes to the package.
#'Accordingly, the package also includes \code{suffix_refresh}, which generates
#'and returns a \emph{fresh} version of the dataset. This can then be passed through
#'to \code{\link{suffix_extract}}.
#'
#'@return a dataset equivalent in format to \code{\link{suffix_dataset}}.
#'
#'@seealso \code{\link{suffix_extract}} to extract suffixes from domain names,
#'or \code{\link{suffix_dataset}} for the inbuilt, default version of the data.
#'
#'@examples
#'\dontrun{
#'new_suffixes <- suffix_refresh()
#'}
#'
#'@export
suffix_refresh <- function(){
  
  has_libcurl <- capabilities("libcurl")
  if(length(has_libcurl) == 0 || has_libcurl == FALSE){
    stop("libcurl support is needed for this function")
  }
  
  #Read in and filter
  connection <- url("https://www.publicsuffix.org/list/effective_tld_names.dat", method = "libcurl")
  results <- readLines(connection, encoding = "UTF-8")
  close(connection)
  
  # making an assumption that sections are broken by blank lines
  blank <- which(results == "")
  # and gotta know where the comments are
  comments <- grep(pattern = "^//", x=results)
  
  # if the file doesn't end on a blank line, stick an ending on there.
  if (blank[length(blank)] < length(results)) {
    blank <- c(blank, length(results)+1)
  }
  # now break up each section into a list
  # grab right after the blank line and right before the next blank line.
  suffix_dataset <- do.call(rbind, lapply(seq(length(blank) - 1), function(i) {
    # these are the lines in the current block
    lines <- seq(blank[i] + 1, blank[i + 1] - 1)
    # assume there is nothing in the block
    rez <- NULL
    # the lines of text in this block
    suff <- results[lines]
    # of which these are the comments
    iscomment <- lines %in% comments
    # and check if we have any results 
    # append the first comment at the top of the block only.
    if(length(suff[!iscomment])) {
      rez <- data.frame(suffixes = suff[!iscomment],
                 comments = suff[which(iscomment)[1]], stringsAsFactors = FALSE)
    }
    return(rez)
  }))
  ## this is the old way
  #suffix_dataset <- results[!grepl(x = results, pattern = "//", fixed = TRUE) & !results == ""]

  #Return the user-friendly version
  return(suffix_dataset)
}

#' @title extract the suffix from domain names
#' @description domain names have suffixes - common endings that people
#' can or could register domains under. This includes things like ".org", but
#' also things like ".edu.co". A simple Top Level Domain list, as a
#' result, probably won't cut it.
#'
#' \code{\link{suffix_extract}} takes the list of public suffixes,
#' as maintained by Mozilla (see \code{\link{suffix_dataset}}) and
#' a vector of domain names, and produces a data.frame containing the
#' suffix that each domain uses, and the remaining fragment.
#'
#' @param domains a vector of damains, from \code{\link{domain}}
#' or \code{\link{url_parse}}. Alternately, full URLs can be provided
#' and will then be run through \code{\link{domain}} internally.
#'
#' @param suffixes a dataset of suffixes. By default, this is NULL and the function
#' relies on \code{\link{suffix_dataset}}. Optionally, if you want more updated
#' suffix data, you can provide the result of \code{\link{suffix_refresh}} for
#' this parameter.
#' 
#' @return a data.frame of four columns, "host" "subdomain", "domain" & "suffix".
#' "host" is what was passed in. "subdomain" is the subdomain of the suffix.
#' "domain" contains the part of the domain name that came before the matched suffix.
#' "suffix" is, well, the suffix.
#'
#' @seealso \code{\link{suffix_dataset}} for the dataset of suffixes.
#'
#' @examples
#'
#' # Using url_parse
#' domain_name <- url_parse("http://en.wikipedia.org")$domain
#' suffix_extract(domain_name)
#'
#' # Using domain()
#' domain_name <- domain("http://en.wikipedia.org")
#' suffix_extract(domain_name)
#'
#' \dontrun{
#' #Relying on a fresh version of the suffix dataset
#' suffix_extract(domain("http://en.wikipedia.org"), suffix_refresh())
#' }
#' 
#' @importFrom triebeard trie longest_match
#' @export
suffix_extract <- function(domains, suffixes = NULL){
  if(!is.null(suffixes)){
    # check if suffixes is a data.frame, and stop if column not found
    if(is.data.frame(suffixes)) {
      if ("suffixes" %in% colnames(suffixes)) {
        suffixes <- suffixes$suffixes
      } else {
        stop("Expected column named \"suffixes\" in suffixes data.frame")
      }
    }
    holding <- suffix_load(suffixes)
  } else {
    holding <- list(suff_trie = urltools_env$suff_trie,
                    is_wildcard = urltools_env$is_wildcard,
                    cleaned_suffixes = urltools_env$cleaned_suffixes)
  }
  
  rev_domains <- reverse_strings(tolower(domains))
  matched_suffixes <- triebeard::longest_match(holding$suff_trie, rev_domains)
  has_wildcard <- matched_suffixes %in% holding$is_wildcard
  is_suffix <- domains %in% holding$cleaned_suffixes
  return(finalise_suffixes(domains, matched_suffixes, has_wildcard, is_suffix))
}

#' @title Dataset of top-level domains (TLDs)
#' @description This dataset contains a registry of top-level domains, as retrieved from
#' and defined by the \href{http://data.iana.org/TLD/tlds-alpha-by-domain.txt}{IANA}.
#' 
#' @docType data
#' @keywords datasets
#' @name tld_dataset
#'
#' @seealso \code{\link{tld_extract}} for extracting TLDs from domain names,
#' and \code{\link{tld_refresh}} to get an updated version of this dataset.
#'
#' @usage data(tld_dataset)
#' @note Last updated 2016-07-20.
#' @format A vector of 1275 elements.
"tld_dataset"

#'@title Retrieve a TLD dataset
#'
#'@description \code{urltools} comes with an inbuilt
#'dataset of top level domains (TLDs), \code{\link{tld_dataset}}.
#'This is used in \code{\link{tld_extract}} to identify the top-level domain
#'within a particular domain name.
#'
#'While updates to the dataset will be included in each new package release,
#'there's going to be a gap between changes to TLDs and changes to the package.
#'Accordingly, the package also includes \code{tld_refresh}, which generates
#'and returns a \emph{fresh} version of the dataset. This can then be passed through
#'to \code{\link{tld_extract}}.
#'
#'@return a dataset equivalent in format to \code{\link{tld_dataset}}.
#'
#'@seealso \code{\link{tld_extract}} to extract suffixes from domain names,
#'or \code{\link{tld_dataset}} for the inbuilt, default version of the data.
#'
#'@examples
#'\dontrun{
#'new_tlds <- tld_refresh()
#'}
#'
#'@export
tld_refresh <- function(){
  raw_tlds <- readLines("http://data.iana.org/TLD/tlds-alpha-by-domain.txt", warn = FALSE)
  raw_tlds <- tolower(raw_tlds[!grepl(x = raw_tlds, pattern = "#", fixed = TRUE)])
  return(raw_tlds)
}

#'@title Extract TLDs
#'@description \code{tld_extract} extracts the top-level domain (TLD) from
#'a vector of domain names. This is distinct from the suffixes, extracted with
#'\code{\link{suffix_extract}}; TLDs are \emph{top} level, while suffixes are just
#'domains through which internet users can publicly register domains (the difference
#'between \code{.org.uk} and \code{.uk}).
#'
#'@param domains a vector of domains, retrieved through \code{\link{url_parse}} or
#'\code{\link{domain}}.
#'
#'@param tlds a dataset of TLDs. If NULL (the default), \code{tld_extract} relies
#'on urltools' \code{\link{tld_dataset}}; otherwise, you can pass in the result of
#'\code{\link{tld_refresh}}.
#'
#'@return a data.frame of two columns: \code{domain}, with the original domain names,
#'and \code{tld}, the identified TLD from the domain.
#'
#'@examples
#'# Using the inbuilt dataset
#'domains <- domain("https://en.wikipedia.org/wiki/Main_Page")
#'tld_extract(domains)
#'
#'# Using a refreshed one
#'tld_extract(domains, tld_refresh())
#'
#'@seealso \code{\link{suffix_extract}} for retrieving suffixes (distinct from TLDs).
#'
#'@export
tld_extract <- function(domains, tlds = NULL){
  if(is.null(tlds)){
    tlds <- urltools::tld_dataset
  }
  guessed_tlds <- tld_extract_(tolower(domains))
  guessed_tlds[!guessed_tlds %in% tlds] <- NA
  return(data.frame(domain = domains, tld = guessed_tlds, stringsAsFactors = FALSE))
}

#'@title Extract hosts
#'@description \code{host_extract} extracts the host from
#'a vector of domain names. A host isn't the same as a domain - it could be
#'the subdomain, if there are one or more subdomains. The host of \code{en.wikipedia.org}
#'is \code{en}, while the host of \code{wikipedia.org} is \code{wikipedia}.
#'
#'@param domains a vector of domains, retrieved through \code{\link{url_parse}} or
#'\code{\link{domain}}.
#'
#'@return a data.frame of two columns: \code{domain}, with the original domain names,
#'and \code{host}, the identified host from the domain.
#'
#'@examples
#'# With subdomains
#'has_subdomain <- domain("https://en.wikipedia.org/wiki/Main_Page")
#'host_extract(has_subdomain)
#'
#'# Without
#'no_subdomain <- domain("https://ironholds.org/projects/r_shiny/")
#'host_extract(no_subdomain)
#'@export
host_extract <- function(domains){
  return(data.frame(domain = domains, host = host_extract_(domains), stringsAsFactors = FALSE))
}

context("Test suffix extraction")

test_that("Suffix extraction works with simple domains",{
  result <- suffix_extract("en.wikipedia.org")
  expect_that(ncol(result), equals(4))
  expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
  expect_that(nrow(result), equals(1))
  
  expect_that(result$subdomain[1], equals("en"))
  expect_that(result$domain[1], equals("wikipedia"))
  expect_that(result$suffix[1], equals("org"))
})

test_that("Suffix extraction works with multiple domains",{
  result <- suffix_extract(c("en.wikipedia.org","en.wikipedia.org"))
  expect_that(ncol(result), equals(4))
  expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
  expect_that(nrow(result), equals(2))
  
  expect_that(result$subdomain[1], equals("en"))
  expect_that(result$domain[1], equals("wikipedia"))
  expect_that(result$suffix[1], equals("org"))
  expect_that(result$subdomain[2], equals("en"))
  expect_that(result$domain[2], equals("wikipedia"))
  expect_that(result$suffix[2], equals("org"))
})

test_that("Suffix extraction works when the domain is the same as the suffix",{
  result <- suffix_extract(c("googleapis.com", "myapi.googleapis.com"))
  expect_that(ncol(result), equals(4))
  expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
  expect_that(nrow(result), equals(2))
  
  expect_equal(result$subdomain[1], NA_character_)
  expect_equal(result$domain[1], NA_character_)
  expect_equal(result$suffix[1], "googleapis.com")
  expect_equal(result$subdomain[2], NA_character_)
  expect_equal(result$domain[2], "myapi")
  expect_equal(result$suffix[2], "googleapis.com")
})

test_that("Suffix extraction works where domains/suffixes overlap", {
  result <- suffix_extract(domain("http://www.converse.com")) # could be se.com or .com
  expect_equal(result$subdomain[1], "www")
  expect_equal(result$domain[1], "converse")
  expect_equal(result$suffix[1], "com")
})

test_that("Suffix extraction works when the domain matches a wildcard suffix",{
  result <- suffix_extract(c("banana.bd", "banana.boat.bd"))
  expect_that(ncol(result), equals(4))
  expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
  expect_that(nrow(result), equals(2))
  
  expect_equal(result$subdomain[1], NA_character_)
  expect_equal(result$domain[1], NA_character_)
  expect_equal(result$suffix[1], "banana.bd")
  expect_equal(result$subdomain[2], NA_character_)
  expect_equal(result$domain[2], "banana")
  expect_equal(result$suffix[2], "boat.bd")
})

test_that("Suffix extraction works when the domain matches a wildcard suffix and has subdomains",{
  result <- suffix_extract(c("foo.bar.banana.bd"))
  expect_that(ncol(result), equals(4))
  expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
  expect_that(nrow(result), equals(1))
  expect_equal(result$subdomain[1], "foo")
  expect_equal(result$domain[1], "bar")
  expect_equal(result$suffix[1], "banana.bd")
})


# test_that("Suffix extraction works with new suffixes",{
#   result <- suffix_extract("en.wikipedia.org", suffix_refresh())
#   expect_that(ncol(result), equals(4))
#   expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
#   expect_that(nrow(result), equals(1))
#   
#   expect_that(result$subdomain[1], equals("en"))
#   expect_that(result$domain[1], equals("wikipedia"))
#   expect_that(result$suffix[1], equals("org"))
# })
# 
# test_that("Suffix extraction works with an arbitrary suffixes database (to ensure it is loading it)",{
#   result <- suffix_extract(c("is-this-a.bananaboat", "en.wikipedia.org"), data.frame(suffixes = "bananaboat"))
#   expect_that(ncol(result), equals(4))
#   expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
#   expect_that(nrow(result), equals(2))
#   
#   expect_equal(result$subdomain[1], NA_character_)
#   expect_equal(result$domain[1], "is-this-a")
#   expect_equal(result$suffix[1], "bananaboat")
#   expect_equal(result$subdomain[2], NA_character_)
#   expect_equal(result$domain[2], NA_character_)
#   expect_equal(result$suffix[2], NA_character_)
# })

test_that("Suffix extraction is back to normal using the internal database when it receives suffixes=NULL",{
  result <- suffix_extract("en.wikipedia.org")
  expect_that(ncol(result), equals(4))
  expect_that(names(result), equals(c("host","subdomain","domain","suffix")))
  expect_that(nrow(result), equals(1))
  
  expect_that(result$subdomain[1], equals("en"))
  expect_that(result$domain[1], equals("wikipedia"))
  expect_that(result$suffix[1], equals("org"))
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#'@title Recompose Parsed URLs
#'
#'@description Sometimes you want to take a vector of URLs, parse them, perform
#'some operations and then rebuild them. \code{url_compose} takes a data.frame produced
#'by \code{\link{url_parse}} and rebuilds it into a vector of full URLs (or: URLs as full
#'as the vector initially thrown into url_parse).
#'
#'This is currently a `beta` feature; please do report bugs if you find them.
#'
#'@param parsed_urls a data.frame sourced from \code{\link{url_parse}}
#'
#'@seealso \code{\link{scheme}} and other accessors, which you may want to
#'run URLs through before composing them to modify individual values.
#'
#'@examples
#'#Parse a URL and compose it
#'url <- "http://en.wikipedia.org"
#'url_compose(url_parse(url))
#'
#'@export
url_compose <- function(parsed_urls) {
    .Call(`_urltools_url_compose`, parsed_urls)
}

#'@title Get or remove user authentication credentials
#'@description authentication credentials appear before the domain
#'name and look like \emph{user:password}. Sometimes you want the removed,
#'or retrieved; \code{strip_credentials} and \code{get_credentials} do
#'precisely that
#'
#'@aliases creds
#'@rdname creds
#'
#'@param urls a URL, or vector of URLs
#'
#'@examples
#'# Remove credentials
#'strip_credentials("http://foo:bar@97.77.104.22:3128")
#'
#'# Get credentials
#'get_credentials("http://foo:bar@97.77.104.22:3128")
#'@export
strip_credentials <- function(urls) {
    .Call(`_urltools_strip_credentials`, urls)
}

#'@rdname creds
#'@export
get_credentials <- function(urls) {
    .Call(`_urltools_get_credentials`, urls)
}

#'@title Encode or decode a URI
#'@description encodes or decodes a URI/URL
#'
#'@param urls a vector of URLs to decode or encode.
#'
#'@details
#'URL encoding and decoding is an essential prerequisite to proper web interaction
#'and data analysis around things like server-side logs. The
#'\href{http://tools.ietf.org/html/rfc3986}{relevant IETF RfC} mandates the percentage-encoding
#'of non-Latin characters, including things like slashes, unless those are reserved.
#'
#'Base R provides \code{\link{URLdecode}} and \code{\link{URLencode}}, which handle
#'URL encoding - in theory. In practise, they have a set of substantial problems
#'that the urltools implementation solves::
#'
#'\itemize{
#' \item{No vectorisation: }{Both base R functions operate on single URLs, not vectors of URLs.
#'       This means that, when confronted with a vector of URLs that need encoding or
#'       decoding, your only option is to loop from within R. This can be incredibly
#'       computationally costly with large datasets. url_encode and url_decode are
#'       implemented in C++ and entirely vectorised, allowing for a substantial
#'       performance improvement.}
#' \item{No scheme recognition: }{encoding the slashes in, say, http://, is a good way
#'       of making sure your URL no longer works. Because of this, the only thing
#'       you can encode in URLencode (unless you refuse to encode reserved characters)
#'       is a partial URL, lacking the initial scheme, which requires additional operations
#'       to set up and increases the complexity of encoding or decoding. url_encode
#'       detects the protocol and silently splits it off, leaving it unencoded to ensure
#'       that the resulting URL is valid.}
#' \item{ASCII NULs: }{Server side data can get very messy and sometimes include out-of-range
#'       characters. Unfortunately, URLdecode's response to these characters is to convert
#'       them to NULs, which R can't handle, at which point your URLdecode call breaks.
#'       \code{url_decode} simply ignores them.}
#'}
#'
#'@return a character vector containing the encoded (or decoded) versions of "urls".
#'
#'@seealso \code{\link{puny_decode}} and \code{\link{puny_encode}}, for punycode decoding
#'and encoding.
#'
#'@examples
#'
#'url_decode("https://en.wikipedia.org/wiki/File:Vice_City_Public_Radio_%28logo%29.jpg")
#'url_encode("https://en.wikipedia.org/wiki/File:Vice_City_Public_Radio_(logo).jpg")
#'
#'\dontrun{
#'#A demonstrator of the contrasting behaviours around out-of-range characters
#'URLdecode("%gIL")
#'url_decode("%gIL")
#'}
#'@rdname encoder
#'@export
url_decode <- function(urls) {
    .Call(`_urltools_url_decode`, urls)
}

#'@rdname encoder
#'@export
url_encode <- function(urls) {
    .Call(`_urltools_url_encode`, urls)
}

#'@title get the values of a URL's parameters
#'@description URLs can have parameters, taking the form of \code{name=value}, chained together
#'with \code{&} symbols. \code{param_get}, when provided with a vector of URLs and a vector
#'of parameter names, will generate a data.frame consisting of the values of each parameter
#'for each URL.
#'
#'@param urls a vector of URLs
#'
#'@param parameter_names a vector of parameter names. If \code{NULL} (default), will extract
#'all parameters that are present.
#'
#'@return a data.frame containing one column for each provided parameter name. Values that
#'cannot be found within a particular URL are represented by an NA.
#'
#'@examples
#'#A very simple example
#'url <- "https://google.com:80/foo.php?this_parameter=selfreferencing&hiphop=awesome"
#'parameter_values <- param_get(url, c("this_parameter","hiphop"))
#'
#'@seealso \code{\link{url_parse}} for decomposing URLs into their constituent parts and
#'\code{\link{param_set}} for inserting or modifying key/value pairs within a query string.
#'
#'@aliases param_get url_parameter
#'@rdname param_get
#'@export
param_get <- function(urls, parameter_names = NULL) {
    .Call(`_urltools_param_get`, urls, parameter_names)
}

#'@title Set the value associated with a parameter in a URL's query.
#'@description URLs often have queries associated with them, particularly URLs for
#'APIs, that look like \code{?key=value&key=value&key=value}. \code{param_set}
#'allows you to modify key/value pairs within query strings, or even add new ones
#'if they don't exist within the URL.
#'
#'@param urls a vector of URLs. These should be decoded (with \code{url_decode})
#'but do not have to have been otherwise manipulated.
#'
#'@param key a string representing the key to modify the value of (or insert wholesale
#'if it doesn't exist within the URL).
#'
#'@param value a value to associate with the key. This can be a single string,
#'or a vector the same length as \code{urls}
#'
#'@return the original vector of URLs, but with modified/inserted key-value pairs. If the
#'URL is \code{NA}, the returned value will be - if the key or value are, no insertion
#'will be made.
#'
#'@examples
#'# Set a URL parameter where there's already a key for that
#'param_set("https://en.wikipedia.org/api.php?action=query", "action", "pageinfo")
#'
#'# Set a URL parameter where there isn't.
#'param_set("https://en.wikipedia.org/api.php?list=props", "action", "pageinfo")
#'
#'@seealso \code{\link{param_get}} to retrieve the values associated with multiple keys in
#'a vector of URLs, and \code{\link{param_remove}} to strip key/value pairs from a URL entirely.
#'
#'@export
param_set <- function(urls, key, value) {
    .Call(`_urltools_param_set`, urls, key, value)
}

#'@title Remove key-value pairs from query strings
#'@description URLs often have queries associated with them, particularly URLs for
#'APIs, that look like \code{?key=value&key=value&key=value}. \code{param_remove}
#'allows you to remove key/value pairs while leaving the rest of the URL intact.
#'
#'@param urls a vector of URLs. These should be decoded with \code{url_decode} but don't
#'have to have been otherwise processed.
#'
#'@param keys a vector of parameter keys to remove.
#'
#'@return the original URLs but with the key/value pairs specified by \code{keys} removed.
#'If the original URL is \code{NA}, \code{NA} will be returned; if a specified key is \code{NA},
#'nothing will be done with it.
#'
#'@seealso \code{\link{param_set}} to modify values associated with keys, or \code{\link{param_get}}
#'to retrieve those values.
#'
#'@examples
#'# Remove multiple parameters from a URL
#'param_remove(urls = "https://en.wikipedia.org/wiki/api.php?action=list&type=query&format=json",
#'             keys = c("action","format"))
#'@export
param_remove <- function(urls, keys) {
    .Call(`_urltools_param_remove`, urls, keys)
}

#'@title split URLs into their component parts
#'@description \code{url_parse} takes a vector of URLs and splits each one into its component
#'parts, as recognised by RfC 3986.
#'
#'@param urls a vector of URLs
#'
#'@details It's useful to be able to take a URL and split it out into its component parts - 
#'for the purpose of hostname extraction, for example, or analysing API calls. This functionality
#'is not provided in base R, although it is provided in \code{\link[httr]{parse_url}}; that
#'implementation is entirely in R, uses regular expressions, and is not vectorised. It's
#'perfectly suitable for the intended purpose (decomposition in the context of automated
#'HTTP requests from R), but not for large-scale analysis.
#'
#'Note that user authentication/identification information is not extracted;
#'this can be found with \code{\link{get_credentials}}.
#'
#'@return a data.frame consisting of the columns scheme, domain, port, path, query
#'and fragment. See the '\href{http://tools.ietf.org/html/rfc3986}{relevant IETF RfC} for
#'definitions. If an element cannot be identified, it is represented by an empty string.
#'
#'@examples
#'url_parse("https://en.wikipedia.org/wiki/Article")
#'
#'@seealso \code{\link{param_get}} for extracting values associated with particular keys in a URL's
#'query string, and \code{\link{url_compose}}, which is \code{url_parse} in reverse.
#'
#'@export
url_parse <- function(urls) {
    .Call(`_urltools_url_parse`, urls)
}

get_component_ <- function(urls, component) {
    .Call(`_urltools_get_component_`, urls, component)
}

set_component_ <- function(urls, component, new_value) {
    .Call(`_urltools_set_component_`, urls, component, new_value)
}

set_component_r <- function(urls, component, new_value, comparator) {
    .Call(`_urltools_set_component_r`, urls, component, new_value, comparator)
}

set_component_f <- function(urls, component, new_value, comparator) {
    .Call(`_urltools_set_component_f`, urls, component, new_value, comparator)
}

rm_component_ <- function(urls, component) {
    .Call(`_urltools_rm_component_`, urls, component)
}

#'@title Encode or Decode Internationalised Domains
#'@description \code{puny_encode} and \code{puny_decode} implement
#'the encoding standard for internationalised (non-ASCII) domains and
#'subdomains. You can use them to encode UTF-8 domain names, or decode
#'encoded names (which start "xn--"), or both.
#'
#'@param x a vector of URLs. These should be URL decoded using \code{\link{url_decode}}.
#'
#'@return a CharacterVector containing encoded or decoded versions of the entries in \code{x}.
#'Invalid URLs (ones that are \code{NA}, or ones that do not successfully map to an actual
#'decoded or encoded version) will be returned as \code{NA}.
#'
#'@examples
#'# Encode a URL
#'puny_encode("https://www.bücher.com/foo")
#'
#'# Decode the result, back to the original
#'puny_decode("https://www.xn--bcher-kva.com/foo")
#'
#'@seealso \code{\link{url_decode}} and \code{\link{url_encode}} for percent-encoding.
#'
#'@rdname puny
#'@export
puny_encode <- function(x) {
    .Call(`_urltools_puny_encode`, x)
}

#'@rdname puny
#'@export
puny_decode <- function(x) {
    .Call(`_urltools_puny_decode`, x)
}

reverse_strings <- function(strings) {
    .Call(`_urltools_reverse_strings`, strings)
}

finalise_suffixes <- function(full_domains, suffixes, wildcard, is_suffix) {
    .Call(`_urltools_finalise_suffixes`, full_domains, suffixes, wildcard, is_suffix)
}

tld_extract_ <- function(domains) {
    .Call(`_urltools_tld_extract_`, domains)
}

host_extract_ <- function(domains) {
    .Call(`_urltools_host_extract_`, domains)
}

context("Test parameter manipulation")

test_that("Parameter parsing can handle multiple, non-existent and pre-trailing parameters",{
  urls <- c("https://www.google.com:80/foo.php?api_params=parsable&this_parameter=selfreferencing&hiphop=awesome",
            "https://www.google.com:80/foo.php?api_params=parsable&this_parameter=selfreferencing&hiphop=awesome#foob",
            "https://www.google.com:80/foo.php?this_parameter=selfreferencing&hiphop=awesome")
  results <- param_get(urls, c("api_params","hiphop"))
  expect_that(results[1:2,1], equals(c("parsable","parsable")))
  expect_true(is.na(results[3,1]))
})

test_that("Parameter parsing works where the parameter appears earlier in the URL", {
  url <- param_get("www.housetrip.es/tos-de-vacaciones/geo?from=01/04/2015&guests=4&to=05/04/2015","to")
  expect_that(ncol(url), equals(1))
  expect_that(url$to[1], equals("05/04/2015"))
})

test_that("Default argument will get all parameter keys", {
  url <- param_get("www.housetrip.es/tos-de-vacaciones/geo?from=01/04/2015&guests=4&to=05/04/2015")
  df <- data.frame("from"="01/04/2015", "guests"="4", "to"="05/04/2015", stringsAsFactors = FALSE)
  testthat::expect_equivalent(url, df)
})

test_that("vectorized get all keys produces NA appropriately", {
  urls <- c("www.housetrip.es/tos-de-vacaciones/geo?guests=4&to=05/04/2015",
            "www.housetrip.es/tos-de-vacaciones/geo?from=01/04/2015&guests=8")

  pars <- param_get(urls)
  df <- data.frame(stringsAsFactors = FALSE,
                   from = c(NA, "01/04/2015"),
                   guests = c("4", "8"),
                   to = c("05/04/2015", NA))

  expect_equivalent(pars, df)
})

test_that("parameter get deals with escaped ampersands and fragments in field values", {
  par <- param_get("http://host/query?foo=bar&amp;baz&amp=nonsense#a=1&b=2", "amp")
  expect_equivalent(par, data.frame(amp="nonsense", stringsAsFactors=FALSE))
  
  par <- param_get("http://host/query?foo=bar&amp;baz&amp=nonsense#a=1&b=2")
  expect_equivalent(par, data.frame(amp="nonsense", foo="bar&amp;baz", stringsAsFactors=FALSE))
})

test_that("Setting parameter values works", {
  expect_true(param_set("https://en.wikipedia.org/wiki/api.php", "baz", "quorn") ==
              "https://en.wikipedia.org/wiki/api.php?baz=quorn")
  expect_true(param_set("https://en.wikipedia.org/wiki/api.php?foo=bar&baz=qux", "baz", "quorn") ==
                "https://en.wikipedia.org/wiki/api.php?foo=bar&baz=quorn")
  expect_true(param_set("https://en.wikipedia.org/wiki/api.php?foo=bar", "baz", "quorn") ==
                "https://en.wikipedia.org/wiki/api.php?foo=bar&baz=quorn")
})

test_that("Setting parameter values quietly fails with NA components", {
  url <- "https://en.wikipedia.org/api.php?action=query"
  expect_identical(url, param_set(url, "action", NA_character_))
  expect_true(is.na(param_set(NA_character_, "action", "foo")))
  expect_identical(url, param_set(url, NA_character_, "pageinfo"))
})

test_that("Setting parameter values works with partially-duplicative keys", {
  url <- "https://en.wikipedia.org/api.php"
  url <- param_set(url, "foo", "bar")
  url <- param_set(url, "oo", "baz")
  testthat::expect_equal(url,
                         "https://en.wikipedia.org/api.php?foo=bar&oo=baz")
})
test_that("Removing parameter entries quietly fails with NA components", {
  url <- "https://en.wikipedia.org/api.php?action=query"
  expect_identical(url, param_remove(url, "foo"))
  expect_true(is.na(param_remove(NA_character_, "action")))
})

test_that("Removing parameter keys works", {
  expect_true(param_remove("https://en.wikipedia.org/api.php?baz=qux", "baz") ==
                "https://en.wikipedia.org/api.php")
})

test_that("Removing parameter keys works when there are multiple parameters in the URL", {
  expect_true(param_remove("https://en.wikipedia.org/api.php?baz=qux&foo=bar", "baz") ==
                "https://en.wikipedia.org/api.php?foo=bar")
})

test_that("Removing parameter keys works when there are multiple parameters to remove", {
  expect_true(param_remove("https://en.wikipedia.org/api.php?baz=qux&foo=bar", c("baz","foo")) ==
                "https://en.wikipedia.org/api.php")
})

test_that("Removing parameter keys works when there is no query", {
  expect_true(param_remove("https://en.wikipedia.org/api.php", "baz") ==
              "https://en.wikipedia.org/api.php")
})

context("Avoid regressions around proxy objects")

test_that("Values are correctly disposed from memory",{
  memfn <- function(d = NULL){
    test_url <- "https://test.com"
    if(!is.null(d)){
      test_url <- urltools::param_set(test_url, "q" , urltools::url_encode(d))
     }
    return(test_url)
  }
  
  baseurl <- "https://test.com"
  expect_equal(memfn(), baseurl)
  expect_equal(memfn("blah"), paste0(baseurl, "?q=blah"))
  expect_equal(memfn(), baseurl)
})

test_that("Parameters correctly add to output",{
  outfn <- function(d = FALSE){
    test_url <- "https://test.com"
    if(d){
      test_url <- urltools::param_set(test_url, "q", urltools::url_encode(d))
    }
    return(test_url)
  }
  
  baseurl <- "https://test.com"
  expect_equal(outfn(), baseurl)
  expect_equal(outfn(TRUE), paste0(baseurl, "?q=TRUE"))
})

context("Check punycode handling")

testthat::test_that("Simple punycode domain encoding works", {
  testthat::expect_identical(puny_encode(enc2utf8("https://www.b\u00FCcher.com/foo")),
                             "https://www.xn--bcher-kva.com/foo")
})

testthat::test_that("Punycode domain encoding works with fragmentary paths", {
  testthat::expect_identical(puny_encode(enc2utf8("https://www.b\u00FCcher.com/")),
                             "https://www.xn--bcher-kva.com/")
})

testthat::test_that("Punycode domain encoding works with ports", {
  testthat::expect_identical(puny_encode(enc2utf8("https://www.b\u00FCcher.com:80")),
                             "https://www.xn--bcher-kva.com:80")
})

testthat::test_that("Punycode domain encoding returns an NA on NAs", {
  testthat::expect_true(is.na(puny_encode(NA_character_)))
})

testthat::test_that("Simple punycode domain decoding works", {
  testthat::expect_identical(puny_decode("https://www.xn--bcher-kva.com/foo"),
                             enc2utf8("https://www.b\u00FCcher.com/foo"))
})

testthat::test_that("Punycode domain decoding works with fragmentary paths", {
  testthat::expect_identical(puny_decode("https://www.xn--bcher-kva.com/"),
                             enc2utf8("https://www.b\u00FCcher.com/"))
})

testthat::test_that("Punycode domain decoding works with ports", {
  testthat::expect_identical(puny_decode("https://www.xn--bcher-kva.com:80"),
                             enc2utf8("https://www.b\u00FCcher.com:80"))
})

testthat::test_that("Punycode domain decoding returns an NA on NAs", {
  testthat::expect_true(is.na(puny_decode(NA_character_)))
})

testthat::test_that("Punycode domain decoding returns an NA on invalid entries", {
  testthat::expect_true(is.na(suppressWarnings(puny_decode("xn--9"))))
})

testthat::test_that("Punycode domain decoding warns on invalid entries", {
  testthat::expect_warning(puny_decode("xn--9"))
})

testthat::context("Test credential extraction and getting")

testthat::test_that("Credentials can be stripped", {
  testthat::expect_identical(strip_credentials("http://foo:bar@97.77.104.22:3128"), "http://97.77.104.22:3128")
})

testthat::test_that("Strings with invalidly-placed credentials are left alone", {
  testthat::expect_identical(strip_credentials("htt@p://foo:bar97.77.104.22:3128"), "htt@p://foo:bar97.77.104.22:3128")
})

testthat::test_that("Invalid URLs are left alone", {
  testthat::expect_identical(strip_credentials("foo:bar@97.77.104.22:3128"), "foo:bar@97.77.104.22:3128")
})

testthat::test_that("Non-objects are left alone", {
  testthat::expect_true(is.na(strip_credentials(NA_character_)))
})

testthat::test_that("Credentials can be retrieved", {
  data <- get_credentials("http://foo:bar@97.43.5421")
  testthat::expect_identical(data$username, "foo")
  testthat::expect_identical(data$authentication, "bar")
})

testthat::test_that("Strings with invalidly-placed credentials are left alone", {
  data <- get_credentials("htt@p://foo:bar97.77.104.22:3128")
  testthat::expect_true(is.na(data$username))
  testthat::expect_true(is.na(data$authentication))
})

testthat::test_that("Invalid URLs are left alone", {
  data <- get_credentials("foo:bar@97.77.104.22:3128")
  testthat::expect_true(is.na(data$username))
  testthat::expect_true(is.na(data$authentication))
})

testthat::test_that("Non-objects are left alone", {
  data <- get_credentials(NA_character_)
  testthat::expect_true(is.na(data$username))
  testthat::expect_true(is.na(data$authentication))
})

context("URL encoding tests")

test_that("Check encoding doesn't encode the scheme", {
  expect_that(url_encode("https://"), equals("https://"))
})

test_that("Check encoding does does not encode pre-path slashes", {
  expect_that(url_encode("https://foo.org/bar/"), equals("https://foo.org/bar%2f"))
})

test_that("Check encoding can handle NAs", {
  expect_that(url_encode(c("https://foo.org/bar/", NA)), equals(c("https://foo.org/bar%2f", NA)))
})

test_that("Check decoding can handle NAs", {
  expect_that(url_decode(c("https://foo.org/bar%2f", NA)), equals(c("https://foo.org/bar/", NA)))
})

# Add comment for windows trickery
test_that("Check decoding and encoding are equivalent", {
  if(.Platform$OS.type == "unix"){
    url <- "Hinrichtung_auf_dem_Altst%c3%a4dter_Ring.JPG%2f120px-Hinrichtung_auf_dem_Altst%c3%a4dter_Ring.JPG"
    decoded_url <- "Hinrichtung_auf_dem_Altst\xc3\xa4dter_Ring.JPG/120px-Hinrichtung_auf_dem_Altstädter_Ring.JPG"
    expect_that((url_decode(url)), equals((decoded_url)))
    expect_that((url_encode(decoded_url)), equals((url)))
  }
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{param_remove}
\alias{param_remove}
\title{Remove key-value pairs from query strings}
\usage{
param_remove(urls, keys)
}
\arguments{
\item{urls}{a vector of URLs. These should be decoded with \code{url_decode} but don't
have to have been otherwise processed.}

\item{keys}{a vector of parameter keys to remove.}
}
\value{
the original URLs but with the key/value pairs specified by \code{keys} removed.
If the original URL is \code{NA}, \code{NA} will be returned; if a specified key is \code{NA},
nothing will be done with it.
}
\description{
URLs often have queries associated with them, particularly URLs for
APIs, that look like \code{?key=value&key=value&key=value}. \code{param_remove}
allows you to remove key/value pairs while leaving the rest of the URL intact.
}
\examples{
# Remove multiple parameters from a URL
param_remove(urls = "https://en.wikipedia.org/wiki/api.php?action=list&type=query&format=json",
            keys = c("action","format"))
}
\seealso{
\code{\link{param_set}} to modify values associated with keys, or \code{\link{param_get}}
to retrieve those values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{port}
\alias{port}
\alias{port<-}
\title{Get or set a URL's port}
\usage{
port(x)

port(x) <- value
}
\arguments{
\item{x}{a URL, or vector of URLs}

\item{value}{a replacement value (or vector of replacement values)
for x's port. If NULL, the port will be entirely removed.}
}
\description{
as in the lubridate package, individual components of a URL
can be both extracted or set using the relevant function call - see the
examples.
}
\examples{
# Get the port
example_url <- "http://cran.r-project.org:80/submit.html"
port(example_url)

# Set the port
port(example_url) <- "12"

# Remove the port
port(example_url) <- NULL
}
\seealso{
\code{\link{scheme}}, \code{\link{domain}}, \code{\link{path}},
\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{domain}
\alias{domain}
\alias{domain<-}
\title{Get or set a URL's domain}
\usage{
domain(x)

domain(x) <- value
}
\arguments{
\item{x}{a URL, or vector of URLs}

\item{value}{a replacement value (or vector of replacement values)
for x's scheme.}
}
\description{
as in the lubridate package, individual components of a URL
can be both extracted or set using the relevant function call - see the
examples.
}
\examples{
#Get a component
example_url <- "http://cran.r-project.org/submit.html"
domain(example_url)

#Set a component
domain(example_url) <- "en.wikipedia.org"
}
\seealso{
\code{\link{scheme}}, \code{\link{port}}, \code{\link{path}},
\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{param_get}
\alias{param_get}
\alias{url_parameter}
\title{get the values of a URL's parameters}
\usage{
param_get(urls, parameter_names = NULL)
}
\arguments{
\item{urls}{a vector of URLs}

\item{parameter_names}{a vector of parameter names. If \code{NULL} (default), will extract
all parameters that are present.}
}
\value{
a data.frame containing one column for each provided parameter name. Values that
cannot be found within a particular URL are represented by an NA.
}
\description{
URLs can have parameters, taking the form of \code{name=value}, chained together
with \code{&} symbols. \code{param_get}, when provided with a vector of URLs and a vector
of parameter names, will generate a data.frame consisting of the values of each parameter
for each URL.
}
\examples{
#A very simple example
url <- "https://google.com:80/foo.php?this_parameter=selfreferencing&hiphop=awesome"
parameter_values <- param_get(url, c("this_parameter","hiphop"))

}
\seealso{
\code{\link{url_parse}} for decomposing URLs into their constituent parts and
\code{\link{param_set}} for inserting or modifying key/value pairs within a query string.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{strip_credentials}
\alias{strip_credentials}
\alias{creds}
\alias{get_credentials}
\title{Get or remove user authentication credentials}
\usage{
strip_credentials(urls)

get_credentials(urls)
}
\arguments{
\item{urls}{a URL, or vector of URLs}
}
\description{
authentication credentials appear before the domain
name and look like \emph{user:password}. Sometimes you want the removed,
or retrieved; \code{strip_credentials} and \code{get_credentials} do
precisely that
}
\examples{
# Remove credentials
strip_credentials("http://foo:bar@97.77.104.22:3128")

# Get credentials
get_credentials("http://foo:bar@97.77.104.22:3128")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suffix.R
\name{suffix_extract}
\alias{suffix_extract}
\title{extract the suffix from domain names}
\usage{
suffix_extract(domains, suffixes = NULL)
}
\arguments{
\item{domains}{a vector of damains, from \code{\link{domain}}
or \code{\link{url_parse}}. Alternately, full URLs can be provided
and will then be run through \code{\link{domain}} internally.}

\item{suffixes}{a dataset of suffixes. By default, this is NULL and the function
relies on \code{\link{suffix_dataset}}. Optionally, if you want more updated
suffix data, you can provide the result of \code{\link{suffix_refresh}} for
this parameter.}
}
\value{
a data.frame of four columns, "host" "subdomain", "domain" & "suffix".
"host" is what was passed in. "subdomain" is the subdomain of the suffix.
"domain" contains the part of the domain name that came before the matched suffix.
"suffix" is, well, the suffix.
}
\description{
domain names have suffixes - common endings that people
can or could register domains under. This includes things like ".org", but
also things like ".edu.co". A simple Top Level Domain list, as a
result, probably won't cut it.

\code{\link{suffix_extract}} takes the list of public suffixes,
as maintained by Mozilla (see \code{\link{suffix_dataset}}) and
a vector of domain names, and produces a data.frame containing the
suffix that each domain uses, and the remaining fragment.
}
\examples{

# Using url_parse
domain_name <- url_parse("http://en.wikipedia.org")$domain
suffix_extract(domain_name)

# Using domain()
domain_name <- domain("http://en.wikipedia.org")
suffix_extract(domain_name)

\dontrun{
#Relying on a fresh version of the suffix dataset
suffix_extract(domain("http://en.wikipedia.org"), suffix_refresh())
}

}
\seealso{
\code{\link{suffix_dataset}} for the dataset of suffixes.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{url_parse}
\alias{url_parse}
\title{split URLs into their component parts}
\usage{
url_parse(urls)
}
\arguments{
\item{urls}{a vector of URLs}
}
\value{
a data.frame consisting of the columns scheme, domain, port, path, query
and fragment. See the '\href{http://tools.ietf.org/html/rfc3986}{relevant IETF RfC} for
definitions. If an element cannot be identified, it is represented by an empty string.
}
\description{
\code{url_parse} takes a vector of URLs and splits each one into its component
parts, as recognised by RfC 3986.
}
\details{
It's useful to be able to take a URL and split it out into its component parts - 
for the purpose of hostname extraction, for example, or analysing API calls. This functionality
is not provided in base R, although it is provided in \code{\link[httr]{parse_url}}; that
implementation is entirely in R, uses regular expressions, and is not vectorised. It's
perfectly suitable for the intended purpose (decomposition in the context of automated
HTTP requests from R), but not for large-scale analysis.

Note that user authentication/identification information is not extracted;
this can be found with \code{\link{get_credentials}}.
}
\examples{
url_parse("https://en.wikipedia.org/wiki/Article")

}
\seealso{
\code{\link{param_get}} for extracting values associated with particular keys in a URL's
query string, and \code{\link{url_compose}}, which is \code{url_parse} in reverse.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{parameters}
\alias{parameters}
\alias{parameters<-}
\title{Get or set a URL's parameters}
\usage{
parameters(x)

parameters(x) <- value
}
\arguments{
\item{x}{a URL, or vector of URLs}

\item{value}{a replacement value (or vector of replacement values)
for x's parameters. If NULL, the parameters will be removed entirely.}
}
\description{
as in the lubridate package, individual components of a URL
can be both extracted or set using the relevant function call - see the
examples.
}
\examples{
# Get the parameters
example_url <- "http://en.wikipedia.org/wiki/Aaron_Halfaker?debug=true"
parameters(example_url)

# Set the parameters
parameters(example_url) <- "debug=false"

# Remove the parameters
parameters(example_url) <- NULL
}
\seealso{
\code{\link{scheme}}, \code{\link{domain}}, \code{\link{port}},
\code{\link{path}} and \code{\link{fragment}} for other accessors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{fragment}
\alias{fragment}
\alias{fragment<-}
\title{Get or set a URL's fragment}
\usage{
fragment(x)

fragment(x) <- value
}
\arguments{
\item{x}{a URL, or vector of URLs}

\item{value}{a replacement value (or vector of replacement values)
for x's fragment. If NULL, the fragment will be removed entirely.}
}
\description{
as in the lubridate package, individual components of a URL
can be both extracted or set using the relevant function call - see the
examples.
}
\examples{
#Get a component
example_url <- "http://en.wikipedia.org/wiki/Aaron_Halfaker?debug=true#test"
fragment(example_url)

#Set a component
fragment(example_url) <- "production"

#Remove a component
fragment(example_url) <- NULL
}
\seealso{
\code{\link{scheme}}, \code{\link{domain}}, \code{\link{port}},
\code{\link{path}} and \code{\link{parameters}} for other accessors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suffix.R
\name{tld_refresh}
\alias{tld_refresh}
\title{Retrieve a TLD dataset}
\usage{
tld_refresh()
}
\value{
a dataset equivalent in format to \code{\link{tld_dataset}}.
}
\description{
\code{urltools} comes with an inbuilt
dataset of top level domains (TLDs), \code{\link{tld_dataset}}.
This is used in \code{\link{tld_extract}} to identify the top-level domain
within a particular domain name.

While updates to the dataset will be included in each new package release,
there's going to be a gap between changes to TLDs and changes to the package.
Accordingly, the package also includes \code{tld_refresh}, which generates
and returns a \emph{fresh} version of the dataset. This can then be passed through
to \code{\link{tld_extract}}.
}
\examples{
\dontrun{
new_tlds <- tld_refresh()
}

}
\seealso{
\code{\link{tld_extract}} to extract suffixes from domain names,
or \code{\link{tld_dataset}} for the inbuilt, default version of the data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{param_set}
\alias{param_set}
\title{Set the value associated with a parameter in a URL's query.}
\usage{
param_set(urls, key, value)
}
\arguments{
\item{urls}{a vector of URLs. These should be decoded (with \code{url_decode})
but do not have to have been otherwise manipulated.}

\item{key}{a string representing the key to modify the value of (or insert wholesale
if it doesn't exist within the URL).}

\item{value}{a value to associate with the key. This can be a single string,
or a vector the same length as \code{urls}}
}
\value{
the original vector of URLs, but with modified/inserted key-value pairs. If the
URL is \code{NA}, the returned value will be - if the key or value are, no insertion
will be made.
}
\description{
URLs often have queries associated with them, particularly URLs for
APIs, that look like \code{?key=value&key=value&key=value}. \code{param_set}
allows you to modify key/value pairs within query strings, or even add new ones
if they don't exist within the URL.
}
\examples{
# Set a URL parameter where there's already a key for that
param_set("https://en.wikipedia.org/api.php?action=query", "action", "pageinfo")

# Set a URL parameter where there isn't.
param_set("https://en.wikipedia.org/api.php?list=props", "action", "pageinfo")

}
\seealso{
\code{\link{param_get}} to retrieve the values associated with multiple keys in
a vector of URLs, and \code{\link{param_remove}} to strip key/value pairs from a URL entirely.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suffix.R
\docType{data}
\name{suffix_dataset}
\alias{suffix_dataset}
\title{Dataset of public suffixes}
\format{A data.frame of 8030 rows and 2 columns}
\usage{
data(suffix_dataset)
}
\description{
This dataset contains a registry of public suffixes, as retrieved from
and defined by the \href{https://publicsuffix.org/}{public suffix list}. It is
sorted by how many periods(".") appear in the suffix, to optimise it for
\code{\link{suffix_extract}}.  It is a data.frame with two columns, the first is
the list of suffixes and the second is our best guess at the comment or owner 
associated with the particular suffix.
}
\note{
Last updated 2016-07-31.
}
\seealso{
\code{\link{suffix_extract}} for extracting suffixes from domain names,
and \code{\link{suffix_refresh}} for getting a new, totally-up-to-date dataset
version.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suffix.R
\docType{data}
\name{tld_dataset}
\alias{tld_dataset}
\title{Dataset of top-level domains (TLDs)}
\format{A vector of 1275 elements.}
\usage{
data(tld_dataset)
}
\description{
This dataset contains a registry of top-level domains, as retrieved from
and defined by the \href{http://data.iana.org/TLD/tlds-alpha-by-domain.txt}{IANA}.
}
\note{
Last updated 2016-07-20.
}
\seealso{
\code{\link{tld_extract}} for extracting TLDs from domain names,
and \code{\link{tld_refresh}} to get an updated version of this dataset.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{puny_encode}
\alias{puny_encode}
\alias{puny_decode}
\title{Encode or Decode Internationalised Domains}
\usage{
puny_encode(x)

puny_decode(x)
}
\arguments{
\item{x}{a vector of URLs. These should be URL decoded using \code{\link{url_decode}}.}
}
\value{
a CharacterVector containing encoded or decoded versions of the entries in \code{x}.
Invalid URLs (ones that are \code{NA}, or ones that do not successfully map to an actual
decoded or encoded version) will be returned as \code{NA}.
}
\description{
\code{puny_encode} and \code{puny_decode} implement
the encoding standard for internationalised (non-ASCII) domains and
subdomains. You can use them to encode UTF-8 domain names, or decode
encoded names (which start "xn--"), or both.
}
\examples{
# Encode a URL
puny_encode("https://www.bücher.com/foo")

# Decode the result, back to the original
puny_decode("https://www.xn--bcher-kva.com/foo")

}
\seealso{
\code{\link{url_decode}} and \code{\link{url_encode}} for percent-encoding.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suffix.R
\name{host_extract}
\alias{host_extract}
\title{Extract hosts}
\usage{
host_extract(domains)
}
\arguments{
\item{domains}{a vector of domains, retrieved through \code{\link{url_parse}} or
\code{\link{domain}}.}
}
\value{
a data.frame of two columns: \code{domain}, with the original domain names,
and \code{host}, the identified host from the domain.
}
\description{
\code{host_extract} extracts the host from
a vector of domain names. A host isn't the same as a domain - it could be
the subdomain, if there are one or more subdomains. The host of \code{en.wikipedia.org}
is \code{en}, while the host of \code{wikipedia.org} is \code{wikipedia}.
}
\examples{
# With subdomains
has_subdomain <- domain("https://en.wikipedia.org/wiki/Main_Page")
host_extract(has_subdomain)

# Without
no_subdomain <- domain("https://ironholds.org/projects/r_shiny/")
host_extract(no_subdomain)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/accessors.R
\name{path}
\alias{path}
\alias{path<-}
\title{Get or set a URL's path}
\usage{
path(x)

path(x) <- value
}
\arguments{
\item{x}{a URL, or vector of URLs}

\item{value}{a replacement value (or vector of replacement values)
for x's path. If NULL, the path will be removed entirely.}
}
\description{
as in the lubridate package, individual components of a URL
can be both extracted or set using the relevant function call - see the
examples.
}
\examples{
# Get the path
example_url <- "http://cran.r-project.org:80/submit.html"
path(example_url)

# Set the path
path(example_url) <- "bin/windows/"

# Remove the path
path(example_url) <- NULL
}
\seealso{
\code{\link{scheme}}, \code{\link{domain}}, \code{\link{port}},
\code{\link{parameters}} and \code{\link{fragment}} for other accessors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{url_decode}
\alias{url_decode}
\alias{url_encode}
\title{Encode or decode a URI}
\usage{
url_decode(urls)

url_encode(urls)
}
\arguments{
\item{urls}{a vector of URLs to decode or encode.}
}
\value{
a character vector containing the encoded (or decoded) versions of "urls".
}
\description{
encodes or decodes a URI/URL
}
\details{
URL encoding and decoding is an essential prerequisite to proper web interaction
and data analysis around things like server-side logs. The
\href{http://tools.ietf.org/html/rfc3986}{relevant IETF RfC} mandates the percentage-encoding
of non-Latin characters, including things like slashes, unless those are reserved.

Base R provides \code{\link{URLdecode}} and \code{\link{URLencode}}, which handle
URL encoding - in theory. In practise, they have a set of substantial problems
that the urltools implementation solves::

\itemize{
\item{No vectorisation: }{Both base R functions operate on single URLs, not vectors of URLs.
      This means that, when confronted with a vector of URLs that need encoding or
      decoding, your only option is to loop from within R. This can be incredibly
      computationally costly with large datasets. url_encode and url_decode are
      implemented in C++ and entirely vectorised, allowing for a substantial
      performance improvement.}
\item{No scheme recognition: }{encoding the slashes in, say, http://, is a good way
      of making sure your URL no longer works. Because of this, the only thing
      you can encode in URLencode (unless you refuse to encode reserved characters)
      is a partial URL, lacking the initial scheme, which requires additional operations
      to set up and increases the complexity of encoding or decoding. url_encode
      detects the protocol and silently splits it off, leaving it unencoded to ensure
      that the resulting URL is valid.}
\item{ASCII NULs: }{Server side data can get very messy and sometimes include out-of-range
      characters. Unfortunately, URLdecode's response to these characters is to convert
      them to NULs, which R can't handle, at which point your URLdecode call breaks.
      \code{url_decode} simply ignores them.}
}
}
\examples{

url_decode("https://en.wikipedia.org/wiki/File:Vice_City_Public_Radio_\%28logo\%29.jpg")
url_encode("https://en.wikipedia.org/wiki/File:Vice_City_Public_Radio_(logo).jpg")

\dontrun{
#A demonstrator of the contrasting behaviours around out-of-range characters
URLdecode("\%gIL")
url_decode("\%gIL")
}
}
\seealso{
\code{\link{puny_decode}} and \code{\link{puny_encode}}, for punycode decoding
and encoding.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/suffix.R
\name{tld_extract}
\alias{tld_extract}
\title{Extract TLDs}
\usage{
tld_extract(domains, tlds = NULL)
}
\arguments{
\item{domains}{a vector of domains, retrieved through \code{\link{url_parse}} or
\code{\link{domain}}.}

\item{tlds}{a dataset of TLDs. If NULL (the default), \code{tld_extract} relies
on urltools' \code{\link{tld_dataset}}; otherwise, you can pass in the result of
\code{\link{tld_refresh}}.}
}
\value{
a data.frame of two columns: \code{domain}, with the original domain names,
and \code{tld}, the identified TLD from the domain.
}
\description{
\code{tld_extract} extracts the top-level domain (TLD) from
a vector of domain names. This is distinct from the suffixes, extracted with
\code{\link{suffix_extract}}; TLDs are \emph{top} level, while suffixes are just
domains through which internet users can publicly register domains (the difference
between \code{.org.uk} and \code{.uk}).
}
\examples{
# Using the inbuilt dataset
domains <- domain("https://en.wikipedia.org/wiki/Main_Page")
tld_extract(domains)

# Using a refreshed one
tld_extract(domains, tld_refresh())

}
\seealso{
\code{\link{suffix_extract}} for retrieving suffixes (distinct from TLDs).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/urltools.R
\docType{package}
\name{urltools}
\alias{urltools}
\alias{urltools-package}
\title{Tools for handling URLs}
\description{
This package provides functions for URL encoding and decoding,
parsing, and parameter extraction, designed to be both fast and
entirely vectorised. It is intended to be useful for people dealing with
web-related datasets, such as server-side logs.
}
\seealso{
the \href{https://CRAN.R-project.org/package=urltools/vignettes/urltools.html}{package vignette}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{url_compose}
\alias{url_compose}
\title{Recompose Parsed URLs}
\usage{
url_compose(parsed_urls)
}
\arguments{
\item{parsed_urls}{a data.frame sourced from \code{\link{url_parse}}}
}
\description{
Sometimes you want to take a vector of URLs, parse them, perform
some operations and then rebuild them. \code{url_compose} takes a data.frame produced
by \code{\link{url_parse}} and rebuilds it into a vector of full URLs (or: URLs as full
as the vector initially thrown into url_parse).

This is currently a `beta` feature; please do report bugs if you find them.
}
\examples{
#Parse a URL and compose it
url <- "http://en.wikipedia.org"
url_compose(url_parse(url))

}
\seealso{
\code{\link{scheme}} and other accessors, which you may want to
run URLs through before composing them to modify individual values.
}
